/* -----------------------------------------------------------------------------------
   This is a small program that shows the basic principles of
   calling HIME functions from a C or C++ program.
   It also shows how to exchange data with HIME.
   It loads HIME.dll dynamically using LoadLibrary.
   
   This program was developed in Dev-C++
   -----------------------------------------------------------------------------------
*/
   
 
#include <windows.h> 
//#include <stdio.h> 
#include <iostream>
#include <stdlib.h>
#include <string.h>
#include <oleauto.h>

using namespace std;

typedef LONG  (__stdcall *HIVERSION)(); 
typedef LONG  (__stdcall *HIREGISTER)(LONG* Reg);
typedef LONG  (__stdcall *HISETOPTION)(LONG* Option, LONG* Param); 
typedef LONG  (__stdcall *HIGETREGLONG)(LONG* Reg); 
typedef LONG  (__stdcall *HIPUTREGLONG)(LONG* Data, LONG* Reg); 
typedef LONG  (__stdcall *HIPUTREGASCIIZ)(LPSTR Data, LONG* Reg);
typedef LONG  (__stdcall *HIGETREGASCIIZ2)(LONG* Reg, char* Data); 
typedef LONG  (__stdcall *HIPUTREG)(BSTR Data, LONG* Reg); 
typedef LONG  (__stdcall *HIPUTREGADD)(BYTE** Address, LONG* Len, LONG* Reg); 
typedef LONG  (__stdcall *HIGETREGADD2)(LONG* Reg, BYTE** Address); 
typedef LONG  (__stdcall *HIREGGETLEN)(LONG* Reg); 
typedef LPSTR (__stdcall *HIGETREGASCIIZ1)(LONG* Reg); 
typedef LONG  (__stdcall *HIDEC2HUGE)(LONG* RegIn, LONG* RegOut); 
typedef LONG  (__stdcall *HIHUGE2DEC)(LONG* RegIn, LONG* RegOut); 
typedef LONG  (__stdcall *HIMUL)(LONG* Reg1, LONG* Reg2, LONG* Result); 


 
int main() 
{ 
    HINSTANCE hinstLib; 
    LONG lResult, lReg1, lReg2, lReg3, lData, lLen, lAddr, lOption;
    LPSTR lpString1, lpString2;
    LPSTR* plpString;
    BSTR bsString;
    BSTR* bspString;
    
    // Get a handle to the DLL
    hinstLib = LoadLibrary(TEXT("HIME.dll"));  
    //hinstLib = LoadLibrary(TEXT("HIME_IOTest.dll"));  
    
    if (hinstLib == NULL)
    {
       cout << ".DLL load FAILED!!";
       system("PAUSE");
       return EXIT_FAILURE;
    }
    
    // If the handle is valid, get the function addresses. 
    HIREGISTER      Hi_Register      =(HIREGISTER)     GetProcAddress(hinstLib, "HI_REGISTER");
    HISETOPTION     Hi_SetOption     =(HISETOPTION)    GetProcAddress(hinstLib, "HI_SETOPTION");
    HIDEC2HUGE      Hi_Dec2Huge      =(HIDEC2HUGE)     GetProcAddress(hinstLib, "HI_DEC2HUGE");    
    HIHUGE2DEC      Hi_Huge2Dec      =(HIHUGE2DEC)     GetProcAddress(hinstLib, "HI_HUGE2DEC");    
    HIMUL           Hi_Mul           =(HIMUL)          GetProcAddress(hinstLib, "HI_MUL");
    HIVERSION       Hi_Version       =(HIVERSION)      GetProcAddress(hinstLib, "HI_VERSION");
    HIGETREGLONG    Hi_GetRegLong    =(HIGETREGLONG)   GetProcAddress(hinstLib, "HI_GETREGLONG");
    HIPUTREGLONG    Hi_PutRegLong    =(HIPUTREGLONG)   GetProcAddress(hinstLib, "HI_PUTREGLONG");
    HIPUTREGASCIIZ  Hi_PutRegAsciiz  =(HIPUTREGASCIIZ) GetProcAddress(hinstLib, "HI_PUTREGASCIIZ");
    HIGETREGASCIIZ2 hi_GetRegAsciiz2 =(HIGETREGASCIIZ2)GetProcAddress(hinstLib, "HI_GETREGASCIIZ2");
    HIGETREGASCIIZ1 hi_GetRegAsciiz1 =(HIGETREGASCIIZ1)GetProcAddress(hinstLib, "HI_GETREGASCIIZ1");
    HIPUTREGADD     Hi_PutRegAdd     =(HIPUTREGADD)    GetProcAddress(hinstLib, "HI_PUTREGADD");
    HIGETREGADD2    Hi_GetRegAdd2    =(HIGETREGADD2)   GetProcAddress(hinstLib, "HI_GETREGADD2");
    HIREGGETLEN     Hi_RegGetLen     =(HIREGGETLEN)    GetProcAddress(hinstLib, "HI_REGGETLEN");

    ////////////// Init HIME //////////////////////////////////////
          //a) 'Register' HIME by entering your license key (if you purchased one).
          //   Otherwise, HIME runs in freeware mode, which shows a nagscreen ocasionally.
    lpString1 = "aaa-bbbbbbbbbb";
    lReg1 = 1;
    lResult = Hi_PutRegAsciiz(lpString1, &lReg1);
    lResult = Hi_Register(&lReg1);
    
    
          //b) Set one or more options
    lOption = 2;
    lData   = 50;   //Set 50 HIME registers      
    lResult = Hi_SetOption(&lOption, &lData);
    if (lResult != 0) cout << "Error: " << lResult << "\n";
    
    
    
        
    ////////////// Exchange data with HIME ////////////////////////
          //a) Store a zero-byte (null) terminated string in a HIME register
    lpString1 = "A string with ascii characters";
    lReg1 = 5;
    lResult = Hi_PutRegAsciiz(lpString1, &lReg1);
    
    cout << "String stored   : " << lpString1 << "\n";
    if (lResult != 0) cout << "Error: " << lResult << "\n";


          //b) Retrieve a zero-byte (null) terminated string from a HIME register
    lReg1 = 5;
    lpString2 = hi_GetRegAsciiz1(&lReg1);
    
    cout << "String retrieved: " << lpString2 << "\n";        
    cout << "with length: " << strlen(lpString2) << "\n";


          //c) Store a byte array in a HIME register. The array can contain binary bytes (null characters).
    BYTE bArr1[] = {65,66,67,68,69,70,71,72,73};
    BYTE *pbArr1;
    pbArr1 = &bArr1[0];
    lLen = sizeof(bArr1);
    lReg1 = 5;            
    lResult = Hi_PutRegAdd(&pbArr1, &lLen, &lReg1);
    
    if (lResult != 0) cout << "Error: " << lResult << "\n";      


          //d) Retrieve a byte array from a HIME register. The array can contain binary bytes (null characters).        
    lReg1 = 5; 
    lLen = Hi_RegGetLen(&lReg1);   //Get number of bytes in HIME register
    cout << "\nHi_RegGetLen returned: " << lLen << "\n";
        
    BYTE bArr2[lLen];
    BYTE *pbArr2;
    pbArr2 = &bArr2[0];
    lResult = Hi_GetRegAdd2(&lReg1, &pbArr2);
    if (lResult != 0) cout << "Error: " << lResult << "\n";
    
    for(int i = 0; i< sizeof(bArr2); i++)
    {
          cout << bArr2[i] << "\n";
    }


          //e) Store a LONG in a HIME register
    lData = 10000;
    lReg1 = 11;
    lResult = Hi_PutRegLong(&lData, &lReg1);
    cout << "\nLONG stored   : " << lData << "\n";
    if (lResult != 0) cout << "Error: " << lResult << "\n";

    
          //f) Retrieve a LONG from a HIME register            
    lReg1 = 11;
    lResult = Hi_GetRegLong(&lReg1);
    cout << "LONG retrieved: " << lResult << "\n";



    ////////////// Do a little math ////////////////////////    
    lReg1     = 1;
    lReg2     = 2;
    lReg3     = 3;
    lpString1 = "1000000000000000000000000000000000000000000000000000000000000000";
    lResult   = Hi_PutRegAsciiz(lpString1, &lReg1);  //Store decimal number in HIME register 1
    lResult   = Hi_Dec2Huge(&lReg1, &lReg1);         //Convert from decimal to 'huge integer' format
    
    lpString1 = "320001";
    lResult   = Hi_PutRegAsciiz(lpString1, &lReg2);  //Store decimal number in HIME register 2
    lResult   = Hi_Dec2Huge(&lReg2, &lReg2);         //Convert from decimal to 'huge integer' format

    lResult   = Hi_Mul(&lReg1, &lReg2, &lReg3);      //Multiply register 1 and 2, store result in register 3
    
    lResult   = Hi_Huge2Dec(&lReg3, &lReg3);         //Convert result from 'huge integer' format to decimal (ascii)
    lpString2 = hi_GetRegAsciiz1(&lReg3);            //Get the result and show it
    cout << "\nResult from multiplication:\n" << lpString2 << "\n";
    
                        
    system("PAUSE");
 
    // Free the DLL.
    lResult = FreeLibrary(hinstLib); 

    
    return EXIT_SUCCESS;
}
